"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var lodash_1 = __importDefault(require("lodash"));
var Operator_1 = require("./Operator");
var Rule = /** @class */ (function () {
    function Rule(definition) {
        if (isCompoundRule(definition)) {
            this.isCompoundRule = true;
            this.and = lodash_1.default.map(definition.and, function (rule) { return new Rule(rule); });
            this.or = lodash_1.default.map(definition.or, function (rule) { return new Rule(rule); });
        }
        else {
            this.isCompoundRule = false;
            Object.assign(this, definition);
        }
    }
    Rule.prototype.toJSON = function () {
        if (this.isCompoundRule) {
            if (lodash_1.default.size(this.and)) {
                return { and: lodash_1.default.map(this.and, function (rule) { return rule.toJSON(); }) };
            }
            else {
                return { or: lodash_1.default.map(this.or, function (rule) { return rule.toJSON(); }) };
            }
        }
        else {
            var _a = this, path = _a.path, operator = _a.operator, value = _a.value;
            return { path: path, operator: operator, value: value };
        }
    };
    /**
     * Evaluates a data object using its configured `path`, `operator`, and
     * `value` properties. Returns a boolean.
     */
    Rule.prototype.evaluate = function (data) {
        var leftOperand = lodash_1.default.get(data, this.path);
        if (this.and) {
            return lodash_1.default.every(this.and, function (rule) { return rule.evaluate(data); });
        }
        if (this.or) {
            return lodash_1.default.every(this.or, function (rule) { return rule.evaluate(data); });
        }
        switch (this.operator) {
            case Operator_1.Operator.IsNull:
                return this.isNil(leftOperand);
            case Operator_1.Operator.IsNotNull:
                return !this.isNil(leftOperand);
            case Operator_1.Operator.IsTrue:
                return leftOperand === true;
            case Operator_1.Operator.IsFalse:
                return leftOperand === false;
            case Operator_1.Operator.Equals:
                return this.equals(leftOperand, this.value);
            case Operator_1.Operator.NotEquals:
                return !this.equals(leftOperand, this.value);
            case Operator_1.Operator.Contains:
                return this.contains(leftOperand, this.value);
            case Operator_1.Operator.NotContains:
                return !this.contains(leftOperand, this.value);
            case Operator_1.Operator.In:
                return this.in(leftOperand, this.value);
            case Operator_1.Operator.NotIn:
                return !this.in(leftOperand, this.value);
            case Operator_1.Operator.GreaterThan:
                return this.greaterThan(leftOperand, this.value);
            case Operator_1.Operator.GreaterThanOrEqualTo:
                return this.greaterThanOrEqualTo(leftOperand, this.value);
            case Operator_1.Operator.LessThan:
                return this.lessThan(leftOperand, this.value);
            case Operator_1.Operator.LessThanOrEqualTo:
                return this.lessThanOrEqualTo(leftOperand, this.value);
            case Operator_1.Operator.ContainsSubstring:
                return this.containsSubstring(leftOperand, this.value);
            case Operator_1.Operator.NotContainsSubstring:
                return !this.containsSubstring(leftOperand, this.value);
        }
    };
    Rule.prototype.isNil = function (value) {
        return lodash_1.default.isNil(value);
    };
    Rule.prototype.equals = function (leftOperand, rightOperand) {
        return lodash_1.default.isEqual(leftOperand, rightOperand);
    };
    Rule.prototype.contains = function (leftOperand, rightOperand) {
        // Support passing in either a single value or an array of values to check.
        rightOperand = lodash_1.default.castArray(rightOperand);
        // `_.intersection` returns the unique values that are included in all given
        // arrays. By checking the size of the resulting array, we can see if any of
        // the values from `rightOperand` are contained within `leftOperand`.
        return !!lodash_1.default.size(lodash_1.default.intersection(leftOperand, rightOperand));
    };
    Rule.prototype.in = function (leftOperand, rightOperand) {
        return lodash_1.default.includes(rightOperand, leftOperand);
    };
    Rule.prototype.greaterThan = function (leftOperand, rightOperand) {
        return lodash_1.default.gt(leftOperand, rightOperand);
    };
    Rule.prototype.greaterThanOrEqualTo = function (leftOperand, rightOperand) {
        return lodash_1.default.gte(leftOperand, rightOperand);
    };
    Rule.prototype.lessThan = function (leftOperand, rightOperand) {
        return lodash_1.default.lt(leftOperand, rightOperand);
    };
    Rule.prototype.lessThanOrEqualTo = function (leftOperand, rightOperand) {
        return lodash_1.default.lte(leftOperand, rightOperand);
    };
    Rule.prototype.containsSubstring = function (leftOperand, rightOperand) {
        if (!leftOperand)
            return false;
        return leftOperand.toLowerCase().includes(rightOperand.toLowerCase());
    };
    return Rule;
}());
exports.Rule = Rule;
function isCompoundRule(rule) {
    return lodash_1.default.has(rule, 'and') || lodash_1.default.has(rule, 'or');
}
